#pragma once

#include <ReferenceCounter.h>
#include <Either.h>

#include "Noise.h"

class Block;

class BiomInterpolator : public virtual Framework::ReferenceCounter
{
public:
    /// <summary>
    /// interpolates two different blocks
    /// the block with a heigher weight has a heigher propability to be returned
    /// the block that is not returned will be deleted
    /// </summary>
    /// <param name="a">the block with weight aWeight</param>
    /// <param name="b">the block with weight bWeight</param>
    /// <param name="aWeight">the weight for block a</param>
    /// <param name="bWeight">the weight for block b</param>
    /// <param name="zNoise">the noise to use</param>
    /// <returns>either a or b</returns>
    virtual Framework::Either<Block*, int> interpolateBlocks( Framework::Either<Block*, int> a, Framework::Either<Block*, int> b, double aWeight, double bWeight, Noise* zNoise ) = 0;
};